@@ -63,13 +63,15 @@
 #include "ivtv-gpio.h"
 
 #include <media/ir-kbd-i2c.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
-#include <media/i2c-compat.h>
-#endif
 
 /* i2c implementation for cx23415/6 chip, ivtv project.
  * Author: Kevin Thayer (nufan_wfk at yahoo.com)
  */
+/* i2c stuff */
+#define IVTV_REG_I2C_SETSCL_OFFSET 0x7000
+#define IVTV_REG_I2C_SETSDA_OFFSET 0x7004
+#define IVTV_REG_I2C_GETSCL_OFFSET 0x7008
+#define IVTV_REG_I2C_GETSDA_OFFSET 0x700c
 
 #ifndef I2C_ADAP_CLASS_TV_ANALOG
 #define I2C_ADAP_CLASS_TV_ANALOG I2C_CLASS_TV_ANALOG
@@ -127,18 +129,6 @@ static const char * const hw_drivernames[] = {
 	"gpio",
 };
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
-static void ivtv_i2c_inc(struct i2c_adapter *adapter)
-{
-	MOD_INC_USE_COUNT;
-}
-
-static void ivtv_i2c_dec(struct i2c_adapter *adapter)
-{
-	MOD_DEC_USE_COUNT;
-}
-#endif
-
 static int attach_inform(struct i2c_client *client)
 {
 	struct ivtv *itv = (struct ivtv *)i2c_get_adapdata(client->adapter);
@@ -154,21 +144,6 @@ static int attach_inform(struct i2c_client *client)
 	if (i == I2C_CLIENTS_MAX) {
 		IVTV_ERR("insufficient room for new I2C client!\n");
 	}
-/* Needs fixing later given sufficient research into the kernels handling
-   of remotes */
-#if 0
-	if (client->addr == 0x7a) {
-		struct IR_i2c *ir = i2c_get_clientdata(client);
-		IVTV_INFO("%s i2c IR detected (%s).\n",
-#if LINUX_VERSION_CODE <=  KERNEL_VERSION(2,6,15)
-				client->driver->name,ir->phys);
-#else
-		client->driver->driver.name, ir->phys);
-#endif
-		ir->get_key   = get_key_pinnacle;
-		ir->ir_codes  = ir_codes_pinnacle;
-	}
-#endif
 	return 0;
 }
 
@@ -195,7 +170,7 @@ static void ivtv_setscl(struct ivtv *itv, int state)
 {
 	/* write them out */
 	/* write bits are inverted */
-	writel(~state, itv->reg_mem + IVTV_REG_I2C_SETSCL_OFFSET);
+	write_reg(~state, IVTV_REG_I2C_SETSCL_OFFSET);
 }
 
 /* Set the serial data line to the desired state */
@@ -203,19 +178,19 @@ static void ivtv_setsda(struct ivtv *itv, int state)
 {
 	/* write them out */
 	/* write bits are inverted */
-	writel(~state & 1, itv->reg_mem + IVTV_REG_I2C_SETSDA_OFFSET);
+	write_reg(~state & 1, IVTV_REG_I2C_SETSDA_OFFSET);
 }
 
 /* Read the serial clock line */
 static int ivtv_getscl(struct ivtv *itv)
 {
-	return readl(itv->reg_mem + IVTV_REG_I2C_GETSCL_OFFSET) & 1;
+	return read_reg(IVTV_REG_I2C_GETSCL_OFFSET) & 1;
 }
 
 /* Read the serial data line */
 static int ivtv_getsda(struct ivtv *itv)
 {
-	return readl(itv->reg_mem + IVTV_REG_I2C_GETSDA_OFFSET) & 1;
+	return read_reg(IVTV_REG_I2C_GETSDA_OFFSET) & 1;
 }
 
 /* Implement a short delay by polling the serial clock line */
@@ -461,7 +436,7 @@ static int ivtv_xfer(struct i2c_adapter *i2c_adap, struct i2c_msg *msgs, int num
 	int retval;
 	int i;
 
-	down(&itv->i2c_bus_lock);
+	mutex_lock(&itv->i2c_bus_lock);
 	for (i = retval = 0; retval == 0 && i < num; i++) {
 		if (msgs[i].flags & I2C_M_RD)
 			retval = ivtv_read(itv, msgs[i].addr, msgs[i].buf, msgs[i].len);
@@ -472,7 +447,7 @@ static int ivtv_xfer(struct i2c_adapter *i2c_adap, struct i2c_msg *msgs, int num
 			retval = ivtv_write(itv, msgs[i].addr, msgs[i].buf, msgs[i].len, stop);
 		}
 	}
-	up(&itv->i2c_bus_lock);
+	mutex_unlock(&itv->i2c_bus_lock);
 	return retval ? retval : num;
 }
 
@@ -495,15 +470,10 @@ static struct i2c_adapter ivtv_i2c_adap_hw_template = {
 	.algo_data = NULL,			/* filled from template */
 	.client_register = attach_inform,
 	.client_unregister = detach_inform,
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
-	.inc_use = ivtv_i2c_inc,	/* inc usage */
-	.dec_use = ivtv_i2c_dec,	/* dec usage */
-#else
 	.owner = THIS_MODULE,
 #ifdef I2C_ADAP_CLASS_TV_ANALOG
 	.class = I2C_ADAP_CLASS_TV_ANALOG,
 #endif
-#endif
 };
 
 static void ivtv_setscl_old(void *data, int state)
@@ -517,7 +487,7 @@ static void ivtv_setscl_old(void *data, int state)
 
         /* write them out */
         /* write bits are inverted */
-        writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSCL_OFFSET);
+        write_reg(~itv->i2c_state, IVTV_REG_I2C_SETSCL_OFFSET);
 }
 
 static void ivtv_setsda_old(void *data, int state)
@@ -531,40 +501,35 @@ static void ivtv_setsda_old(void *data, int state)
 
         /* write them out */
         /* write bits are inverted */
-        writel(~itv->i2c_state, itv->reg_mem + IVTV_REG_I2C_SETSDA_OFFSET);
+        write_reg(~itv->i2c_state, IVTV_REG_I2C_SETSDA_OFFSET);
 }
 
 static int ivtv_getscl_old(void *data)
 {
         struct ivtv *itv = (struct ivtv *)data;
 
-        return readb(itv->reg_mem + IVTV_REG_I2C_GETSCL_OFFSET);
+        return read_reg(IVTV_REG_I2C_GETSCL_OFFSET) & 1;
 }
 
 static int ivtv_getsda_old(void *data)
 {
         struct ivtv *itv = (struct ivtv *)data;
 
-        return readb(itv->reg_mem + IVTV_REG_I2C_GETSDA_OFFSET);
+        return read_reg(IVTV_REG_I2C_GETSDA_OFFSET) & 1;
 }
 
 /* template for i2c-bit-algo */
 static struct i2c_adapter ivtv_i2c_adap_template = {
         .name = "ivtv i2c driver",
-        .id = I2C_HW_B_BT848,      /* algo-bit is OR'd with this */
-	.algo = NULL,                           /* set by i2c-algo-bit */
-        .algo_data = NULL,                      /* filled from template */
+        .id = I2C_HW_B_CX2341X,  	/* algo-bit is OR'd with this */
+	.algo = NULL,                   /* set by i2c-algo-bit */
+        .algo_data = NULL,              /* filled from template */
         .client_register = attach_inform,
         .client_unregister = detach_inform,
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
-	.inc_use = ivtv_i2c_inc,	/* inc usage */
-	.dec_use = ivtv_i2c_dec,	/* dec usage */
-#else
 	.owner = THIS_MODULE,
 #ifdef I2C_ADAP_CLASS_TV_ANALOG
 	.class = I2C_ADAP_CLASS_TV_ANALOG,
 #endif
-#endif
 };
 
 static struct i2c_algo_bit_data ivtv_i2c_algo_template = {
@@ -573,8 +538,10 @@ static struct i2c_algo_bit_data ivtv_i2c_algo_template = {
         ivtv_setscl_old,        /* " */
         ivtv_getsda_old,        /* " */
         ivtv_getscl_old,        /* " */
-        10,                     /* udelay or mdelay */
-        10,                     /* whatever above isn't */
+        10,                     /* udelay */
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18)
+	10,
+#endif
         200                     /* timeout */
 };
 
@@ -589,7 +556,6 @@ static int ivtv_call_i2c_client(struct ivtv *itv, int addr, unsigned int cmd,
 	int retval;
 	int i;
 
-	down(&itv->i2c_lock);
 	IVTV_DEBUG_I2C("call_i2c_client addr=%02x\n", addr);
 	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
 		client = itv->i2c_clients[i];
@@ -601,12 +567,10 @@ static int ivtv_call_i2c_client(struct ivtv *itv, int addr, unsigned int cmd,
 		}
 		if (addr == client->addr) {
 			retval = client->driver->command(client, cmd, arg);
-			up(&itv->i2c_lock);
 			return retval;
 		}
 	}
 	IVTV_ERR("i2c addr 0x%02x not found for command 0x%x!\n", addr, cmd);
-	up(&itv->i2c_lock);
 	return -ENODEV;
 }
 
@@ -618,7 +582,6 @@ int ivtv_i2c_id_addr(struct ivtv *itv, u32 id)
 	int retval = -ENODEV;
 	int i;
 
-	down(&itv->i2c_lock);
 	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
 		client = itv->i2c_clients[i];
 		if (client == NULL)
@@ -628,7 +591,6 @@ int ivtv_i2c_id_addr(struct ivtv *itv, u32 id)
 			break;
 		}
 	}
-	up(&itv->i2c_lock);
 	return retval;
 }
 
@@ -771,6 +733,9 @@ int __devinit init_ivtv_i2c(struct ivtv *itv)
 	memcpy(&itv->i2c_client, &ivtv_i2c_client_template,
 	       sizeof(struct i2c_client));
 	itv->i2c_client.adapter = &itv->i2c_adap;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+        itv->i2c_adap.dev.parent = &itv->dev->dev;
+#endif
   
 	IVTV_DEBUG_I2C("setting scl and sda to 1\n");
 	ivtv_setscl(itv, 1);
@@ -786,9 +751,13 @@ void __devexit exit_ivtv_i2c(struct ivtv *itv)
 {
 	IVTV_DEBUG_I2C("i2c exit\n");
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+        i2c_del_adapter(&itv->i2c_adap);
+#else
         if (itv->options.newi2c > 0) {
                 i2c_del_adapter(&itv->i2c_adap);
         } else {
                 i2c_bit_del_bus(&itv->i2c_adap);
         }
+#endif
 }
