@@ -21,15 +21,10 @@ int ivtv_streams_setup(struct ivtv *itv);
 void ivtv_streams_cleanup(struct ivtv *itv);
 
 /* Capture related */
-void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type);
-int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type);
+int ivtv_start_v4l2_encode_stream(struct ivtv_stream *s);
+int ivtv_start_v4l2_decode_stream(struct ivtv_stream *s, int gop_offset);
+int ivtv_stop_v4l2_decode_stream(struct ivtv_stream *s, int flags, u64 pts);
 
-int ivtv_start_v4l2_decode_stream(struct ivtv *itv, int type);
-int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type);
-int ivtv_stop_v4l2_decode_stream(struct ivtv *itv, int type);
-
-int ivtv_stop_capture_stream(struct ivtv *itv, int type);
-int ivtv_stop_capture(struct ivtv *itv, int type);
-int ivtv_stop_all_captures(struct ivtv *itv);
+int ivtv_stop_capture(struct ivtv_stream *s);
+void ivtv_stop_all_captures(struct ivtv *itv);
 int ivtv_passthrough_mode(struct ivtv *itv, int enable);
-
