@@ -16,6 +16,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-int ivtv_yuv_prep_frame(struct ivtv *itv,
-			       struct ivtvyuv_ioctl_dma_host_to_ivtv_args *args);
-void ivtv_yuv_filter(struct ivtv *itv, int h_filter, int v_filter_1, int v_filter_2);
+int ivtv_yuv_filter_check(struct ivtv *itv);
+int ivtv_yuv_prep_frame(struct ivtv *itv, struct ivtvyuv_ioctl_dma_host_to_ivtv_args *args);
+void ivtv_yuv_close(struct ivtv *itv);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+void ivtv_yuv_work_handler (struct work_struct *work);
+#else
+void ivtv_yuv_work_handler (void *arg);
+#endif
