@@ -34,15 +34,21 @@
 #include <math.h>
 #include <linux/types.h>
 
-#define __user
-#include <linux/videodev2.h>
+#include "videodev2.h"
 
 #include "ivtv.h"
 
+struct v4l2_register {
+	__u32 i2c_id; 		/* I2C driver ID of the I2C chip. 0 for the I2C adapter. */
+	unsigned long reg;
+	__u32 val;
+};
+#define	VIDIOC_INT_G_REGISTER 		_IOWR('d', 101, struct v4l2_register)
+#define I2C_DRIVERID_CX25840	71	/* cx2584x video encoder	*/
+#define I2C_DRIVERID_SAA711X	73	/* saa711x video encoders	*/
+#define I2C_DRIVERID_SAA717X	80	/* saa717x video encoder	*/
 #define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? '.' : ((c) & 0x7F))
 
-int valid_char(char c);
-
 int valid_char(char c)
 {
 	/* Invalid Character */
@@ -56,6 +62,10 @@ int frames = 0;
 int lines = 0;
 int space_needed = 0;
 int text_off = 0;
+int page = -1;
+int show_wss = 1;
+int show_ttx = 1;
+int show_vps = 1;
 
 static const char *formats[] = {
 	"Full format 4:3, 576 lines",
@@ -74,6 +84,56 @@ static const char *subtitles[] = {
 	"?"
 };
 
+/* ETS 300 706, Section 8.2 Hamming 8/4 */
+/* Hamming code copied from zvbi */
+const int8_t
+_vbi_hamm8_inv [256] = {
+	0x01, 0xff, 0x01, 0x01, 0xff, 0x00, 0x01, 0xff, 
+	0xff, 0x02, 0x01, 0xff, 0x0a, 0xff, 0xff, 0x07, 
+	0xff, 0x00, 0x01, 0xff, 0x00, 0x00, 0xff, 0x00, 
+	0x06, 0xff, 0xff, 0x0b, 0xff, 0x00, 0x03, 0xff, 
+
+	0xff, 0x0c, 0x01, 0xff, 0x04, 0xff, 0xff, 0x07, 
+	0x06, 0xff, 0xff, 0x07, 0xff, 0x07, 0x07, 0x07, 
+	0x06, 0xff, 0xff, 0x05, 0xff, 0x00, 0x0d, 0xff, 
+	0x06, 0x06, 0x06, 0xff, 0x06, 0xff, 0xff, 0x07, 
+
+	0xff, 0x02, 0x01, 0xff, 0x04, 0xff, 0xff, 0x09, 
+	0x02, 0x02, 0xff, 0x02, 0xff, 0x02, 0x03, 0xff, 
+	0x08, 0xff, 0xff, 0x05, 0xff, 0x00, 0x03, 0xff, 
+	0xff, 0x02, 0x03, 0xff, 0x03, 0xff, 0x03, 0x03, 
+
+	0x04, 0xff, 0xff, 0x05, 0x04, 0x04, 0x04, 0xff, 
+	0xff, 0x02, 0x0f, 0xff, 0x04, 0xff, 0xff, 0x07, 
+	0xff, 0x05, 0x05, 0x05, 0x04, 0xff, 0xff, 0x05, 
+	0x06, 0xff, 0xff, 0x05, 0xff, 0x0e, 0x03, 0xff, 
+
+	0xff, 0x0c, 0x01, 0xff, 0x0a, 0xff, 0xff, 0x09, 
+	0x0a, 0xff, 0xff, 0x0b, 0x0a, 0x0a, 0x0a, 0xff, 
+	0x08, 0xff, 0xff, 0x0b, 0xff, 0x00, 0x0d, 0xff, 
+	0xff, 0x0b, 0x0b, 0x0b, 0x0a, 0xff, 0xff, 0x0b, 
+
+	0x0c, 0x0c, 0xff, 0x0c, 0xff, 0x0c, 0x0d, 0xff, 
+	0xff, 0x0c, 0x0f, 0xff, 0x0a, 0xff, 0xff, 0x07, 
+	0xff, 0x0c, 0x0d, 0xff, 0x0d, 0xff, 0x0d, 0x0d, 
+	0x06, 0xff, 0xff, 0x0b, 0xff, 0x0e, 0x0d, 0xff, 
+
+	0x08, 0xff, 0xff, 0x09, 0xff, 0x09, 0x09, 0x09, 
+	0xff, 0x02, 0x0f, 0xff, 0x0a, 0xff, 0xff, 0x09, 
+	0x08, 0x08, 0x08, 0xff, 0x08, 0xff, 0xff, 0x09, 
+	0x08, 0xff, 0xff, 0x0b, 0xff, 0x0e, 0x03, 0xff, 
+
+	0xff, 0x0c, 0x0f, 0xff, 0x04, 0xff, 0xff, 0x09, 
+	0x0f, 0xff, 0x0f, 0x0f, 0xff, 0x0e, 0x0f, 0xff, 
+	0x08, 0xff, 0xff, 0x05, 0xff, 0x0e, 0x0d, 0xff, 
+	0xff, 0x0e, 0x0f, 0xff, 0x0e, 0x0e, 0xff, 0x0e
+};
+
+static int vbi_unham8(unsigned int c)
+{
+	return _vbi_hamm8_inv[(uint8_t) c];
+}
+
 void decode_wss(struct v4l2_sliced_vbi_data *s)
 {
 	unsigned char parity;
@@ -389,6 +449,52 @@ void decode_vps(struct v4l2_sliced_vbi_data *s)
 	dump_pil(pil);
 }
 
+static void teletext(struct v4l2_sliced_vbi_data *s)
+{
+	unsigned char mag = (vbi_unham8(s->data[1]) << 4) | vbi_unham8(s->data[0]);
+	static int last_page[8];
+	static int last_page1;
+	static int sub = 0;
+	static int inh = 0;
+	static int ser = 0;
+	int i = 2;
+	
+	if ((mag >> 3) == 0) {
+		unsigned char mg = mag & 7;
+		unsigned short pg = (vbi_unham8(s->data[3]) << 4) | vbi_unham8(s->data[2]);
+
+		if (mg == 0) mg = 8;
+		pg += mg * 256;
+		last_page1 = last_page[mag & 7] = pg;
+		if (page != -1 && pg != page) {
+			return;
+		}
+		i = 10;
+	}
+	else if (page != -1 && (ser ? last_page1 : last_page[mag & 7]) != page)
+		return;
+	if ((mag >> 3) == 0) {
+		unsigned char c = vbi_unham8(s->data[9]);
+		ser = c * 1;
+		c = vbi_unham8(s->data[7]);
+		sub = c & 8;
+		c = vbi_unham8(s->data[8]);
+		inh = c & 8;
+		if (c & 1)
+			return;
+	}
+	if (inh)
+		return;
+	if ((mag >> 3) > 24)
+		return;
+	while (i < 42) {
+		int c = s->data[i] & 0x7f;
+		printf("%c", c >= 32 ? c : ' ');
+		i++;
+	}
+	printf("\n");
+}
+
 void process(struct v4l2_sliced_vbi_data *s)
 {
 	if (s->id == 0)
@@ -397,17 +503,20 @@ void process(struct v4l2_sliced_vbi_data *s)
         //printf("%04d: line %02u field %d type %x\n", frames, s->line, s->field, s->id);
 	switch (s->id) {
 	case V4L2_SLICED_TELETEXT_B:
-		printf("teletext\n");
+		if (show_ttx)
+			teletext(s);
 		break;
 	case V4L2_SLICED_VPS:
                 if (s->line != 16 || s->field)
                         break;
-		decode_vps(s);
+		if (show_vps)
+			decode_vps(s);
 		break;
 	case V4L2_SLICED_WSS_625:
                 if (s->line != 23 || s->field)
                         break;
-		decode_wss(s);
+		if (show_wss)
+			decode_wss(s);
 		break;
 	case V4L2_SLICED_CAPTION_525:
                 if (s->line != 21)
@@ -425,6 +534,62 @@ static void sigalrm(int s)
 	exit(0);
 }
 
+static void cc_readback(int fh)
+{
+	struct v4l2_sliced_vbi_data data;
+	struct v4l2_register v;
+	int parity[2];
+	int cc_cnt = 0;
+	int field = -1;
+
+	v.i2c_id = I2C_DRIVERID_CX25840;
+	v.reg = 0x444;
+	if (ioctl(fh, VIDIOC_INT_G_REGISTER, &v))
+		v.i2c_id = I2C_DRIVERID_SAA711X;
+
+	while (1) {
+		if (v.i2c_id == I2C_DRIVERID_CX25840) {
+			v.reg = 0x444;
+			ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+			if ((v.val & 32) == 32) {
+				int skip = v.val;
+				v.reg = 0x445;
+				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				if (skip & 16)
+					continue;
+				parity[cc_cnt] = v.val & 128;
+				data.data[cc_cnt++] = v.val;
+				if (cc_cnt == 2) {
+					data.field = 0;
+					if (odd_parity(data.data[0] &&
+								odd_parity(data.data[1])))
+						decode_cc(&data);
+					cc_cnt = 0;
+				}
+				continue;
+			}
+		}
+		else {
+			v.reg = 0x66;
+			ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+			if ((v.val & 0x30) == 0 && (v.val & 0xe) != field) {
+				v.reg = 0x69;
+				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				data.data[0] = v.val;
+				v.reg = 0x6a;
+				ioctl(fh, VIDIOC_INT_G_REGISTER, &v);
+				data.data[1] = v.val;
+				data.field = 0;
+				if (odd_parity(data.data[0] && odd_parity(data.data[1])))
+					decode_cc(&data);
+				continue;
+			}
+		}
+		struct timeval tv = { 0, 1000 };
+		select(0, NULL, NULL, NULL, &tv);
+	}
+}
+
 int main(int argc, char **argv)
 {
 	char device[100] = "/dev/vbi0";
@@ -432,29 +597,42 @@ int main(int argc, char **argv)
         v4l2_std_id std;
 	struct v4l2_sliced_vbi_data *buf;
 	int timeout = 0;
+	int readback = 0;
 	int fh;
 
 	while (1) {
-		int ch = getopt(argc, argv, "d:t:h");
+		int ch = getopt(argc, argv, "d:t:hp:rs:");
 
 		if (ch == -1)
 			break;
 		switch (ch) {
 		case 'd':
-		        strncpy(device, optarg , 99 );
-			device[99] = 0;
-			if (strlen(optarg) > 99 ) {
-				fprintf(stderr, "device too long [max 99 char], truncated to %s" , device);
-			}
+			strcpy(device, optarg);
 			if (optarg[0] >= '0' && optarg[0] <= '9')
 				sprintf(device, "/dev/vbi%s", optarg);
 			break;
 		case 't':
 			timeout = atol(optarg);
 			break;
+		case 'p':
+			page = atol(optarg);
+			page = ((page / 100) << 8) | (((page / 10) % 10) << 4) | (page % 10);
+			break;
+		case 'r':
+			readback = 1;
+			break;
+		case 's':
+			show_ttx = strstr(optarg, "teletext") != NULL;
+			show_vps = strstr(optarg, "vps") != NULL;
+			show_wss = strstr(optarg, "wss") != NULL;
+			break;
 		default:
-			fprintf(stderr, "usage: %s [-d <dev>] [-t <secs>] [-h]\n"
-					"defaults are /dev/vbi0 and no timeout\n", argv[0]);
+			fprintf(stderr, "usage: %s [-d <dev>] [-t <secs>] [-p <page>] [-s teletext,wss,vps] [-r] [-h]\n"
+					"defaults are /dev/vbi0 and no timeout\n"
+					"-p selects teletext page\n"
+					"-r switches to using the readback registers on the digitizer\n"
+					"-s selects what to show\n",
+				       	argv[0]);
 			exit(-1);
 		}
 	}
@@ -479,11 +657,16 @@ int main(int argc, char **argv)
 	}
 
 	fprintf(stderr, "%08x, %d\n", fmt.fmt.sliced.service_set, fmt.fmt.sliced.io_size);
-	buf = malloc(fmt.fmt.sliced.io_size);
 	if (timeout) {
 		signal(SIGALRM, sigalrm);
 		alarm(timeout);
 	}
+	if (readback) {
+		cc_readback(fh);
+		close(fh);
+		return 0;
+	}
+	buf = malloc(fmt.fmt.sliced.io_size);
 	for (;;) {
 		int size = read(fh, buf, fmt.fmt.sliced.io_size);
 		int i;
