@@ -37,8 +37,7 @@
 #include <linux/types.h>
 #include <sys/klog.h>
 
-#define __user
-#include <linux/videodev2.h>
+#include "videodev2.h"
 
 /* Internals copied from media/v4l2-common.h */
 struct v4l2_routing {
@@ -131,15 +130,6 @@ static void usage(void)
 	printf("  -h, --help         display this help message\n");
 	printf("  -K, --passthrough <mode>\n");
 	printf("                     set passthrough mode: 1 = on, 0 = off [IVTV_IOC_PASSTHROUGH]\n");
-	printf("  -W, --get-vbi-passthrough\n");
-	printf("                     query the current VBI passthrough mode [IVTV_IOC_G_VBI_PASSTHROUGH]\n");
-	printf("  -w, --set-vbi-passthrough=<mode>\n");
-	printf("                     set the current VBI passthrough mode to <mode> [IVTV_IOC_S_VBI_PASSTHROUGH]\n");
-	printf("                     <mode> is a comma separated list of:\n");
-	printf("                     none: no passthrough\n");
-	printf("                     cc:   closed caption\n");
-	printf("                     wss:  widescreen signal\n");
-	printf("                     vps:  vps\n");
 	printf("  --get-yuv-mode     display the current yuv mode\n");
 	printf("  --set-yuv-mode [param],[param]\n");
         printf("                     set the current yuv mode\n");
@@ -152,9 +142,11 @@ static void usage(void)
 	printf("\n");
 	printf("Expert options:\n");
 	printf("  -D, --set-debug <level>\n");
-	printf("                     set the module ivtv_debug variable\n");
+	printf("                     set the module debug variable\n");
+	printf("                      1: WARN   2: INFO  4: API   8: DMA\n");
+	printf("                     16: IOCTL 32: I2C  64: IRQ 128: DEC 256: YUV\n");
 	printf("  -E, --end-gop      capture last GOP [IVTV_IOC_S_GOP_END]\n");
-	printf("  -e, --get-debug    query the module ivtv_debug variable\n");
+	printf("  -e, --get-debug    query the module debug variable\n");
 	printf("  -I, --list-gpio\n");
 	printf("                     show GPIO input/direction/output bits\n");
 	printf("  -i, --set-gpio [dir=<dir>,]val=<val>\n");
@@ -283,25 +275,6 @@ static int doioctl(int fd, int request, void *parm, const char *name)
 	
 	return retVal;
 }
-/*
-static void printregs(int fd, int start, int length)
-{
-	struct ivtv_ioctl_register saa7115_reg;
-        int i;
-
-	printf("\n      00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
-        for (i = start; i < start + length; i++) {
-                saa7115_reg.reg = i;
-                if (ioctl(fd, IVTV_IOC_G_DECODER_REG, &saa7115_reg) < 0)
-                        fprintf(stderr, "ioctl: IVTV_IOC_G_DECODER_REG failed\n");
-                else {
-                        if ((i & 0xf) == 0)
-                                printf("\n%04x: ", i);
-                        printf("%02x ", saa7115_reg.val);
-                }
-        }
-        printf("\n");
-}*/
 
 /* Copied from <linux/i2c-id.h> */
 #define I2C_DRIVERID_SAA711X	73	/* saa711x video encoders	*/
@@ -324,6 +297,40 @@ static int parse_chip(const char *chip)
 	return -1;
 }
 
+static void print_regs(int fd, struct v4l2_register *reg, unsigned long min, unsigned long max, int stride)
+{
+	unsigned long mask = stride > 1 ? 0x1f : 0x0f;
+	unsigned long i;
+	int line = 0;
+
+	for (i = min & ~mask; i <= max; i += stride) {
+		if ((i & mask) == 0 && line % 32 == 0) {
+			if (stride == 4)
+				printf("\n                00       04       08       0C       10       14       18       1C");
+			else
+				printf("\n          00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
+		}
+	
+		if ((i & mask) == 0) {
+			printf("\n%08lx: ", i);
+			line++;
+		}
+		if (i < min) {
+			printf("%*s ", 2 * stride, "");
+			continue;
+		}
+		reg->reg = i;
+		if (ioctl(fd, VIDIOC_INT_G_REGISTER, reg) < 0) {
+			fprintf(stderr, "ioctl: VIDIOC_INT_G_REGISTER "
+					"failed for 0x%08lx\n", reg->reg);
+		} else {
+			printf("%0*x ", 2 * stride, reg->val);
+		}
+		usleep(1);
+	}
+	printf("\n");
+}
+
 int main(int argc, char **argv)
 {
 	char *value, *subs;
@@ -359,11 +366,12 @@ int main(int argc, char **argv)
 	unsigned int set_yuvMode = 0;
 
 	/* command args */
-	char ch, *device = strdup("/dev/video0");	/* -d device */
+	char *device = strdup("/dev/video0");	/* -d device */
+        int ch;
 	struct ivtv_ioctl_yuv_interlace yuv_mode = { 0, 0 };
 	struct v4l2_register reg;
-	unsigned long min_reg = 0x00000000;	/* max registers to list */
-	unsigned long max_reg = 0x0200ffff;	/* max registers to list */
+	unsigned long min_reg = 0;	/* min registers to list */
+	unsigned long max_reg = 0;	/* max registers to list: 0 == undefined */
 	struct v4l2_routing route;	/* audio_route */
 	unsigned short gpio_out = 0x0;	/* GPIO output data */
 	unsigned short gpio_dir = 0x0;	/* GPIO direction bits */
@@ -556,6 +564,12 @@ int main(int argc, char **argv)
 					break;
 				}
 			}
+			if (min_reg > max_reg) {
+				fprintf(stderr, "Invalid suboptions specified\n");
+				usage();
+				exit(1);
+				break;
+			}
 			break;
 		case OptSetAudioRoute:
 			subs = optarg;
@@ -651,20 +665,6 @@ int main(int argc, char **argv)
 
 	/* Setting Opts */
 
-	if (options[OptSetVbiPassThrough]) {
-		if (doioctl(fd, IVTV_IOC_S_VBI_PASSTHROUGH, &vbi_passthrough, 
-			"IVTV_IOC_S_VBI_PASSTHROUGH") == 0)
-			printf("VBI passthrough mode set to %08x\n",
-			       vbi_passthrough);
-	}
-
-	if (options[OptGetVbiPassThrough]) {
-		if (doioctl(fd, IVTV_IOC_G_VBI_PASSTHROUGH, &vbi_passthrough,
-			"IVTV_IOC_G_VBI_PASSTHROUGH") == 0)
-			printf("VBI passthrough mode is %08x\n",
-			       vbi_passthrough);
-	}
-
 	if (options[OptEndGop]) {
 		if (doioctl(fd, IVTV_IOC_S_GOP_END, &end_gop,
 			"IVTV_IOC_S_GOP_END") == 0)
@@ -687,29 +687,33 @@ int main(int argc, char **argv)
 
 		if (reg.i2c_id == 0) stride = 4;
 		printf("ioctl: VIDIOC_INT_G_REGISTER\n");
-		if (stride == 4)
-			printf("                00       04       08       0C       10       14       18       1C");
-		else
-			printf("          00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
-		for (i = 0; i <= max_reg - min_reg; i += stride) {
-			reg.reg = min_reg + (unsigned long)i;
-			if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) < 0) {
-				fprintf(stderr,
-					"ioctl: "
-					"VIDIOC_INT_G_REGISTER "
-					"failed for 0x%08lx\n", reg.reg);
-			} else {
-				/*if ((i&0xf)==0) 
-				   printf("\n%08x: ",i); */
-				if (stride > 1 && (i & 0x1f) == 0)
-					printf("\n%08x: ", i);
-				else if (stride == 1 && (i & 0x0f) == 0)
-					printf("\n%08x: ", i);
-				printf("%0*x ", 2 * stride, reg.val);
+		if (max_reg == 0) {
+			switch (reg.i2c_id) {
+			case I2C_DRIVERID_SAA711X:
+				print_regs(fd, &reg, 0, 0xff, stride);
+				break;
+			case I2C_DRIVERID_SAA717X:
+				// FIXME: use correct reg regions
+				print_regs(fd, &reg, 0, 0xff, stride);
+				break;
+			case I2C_DRIVERID_SAA7127:
+				print_regs(fd, &reg, 0, 0x7f, stride);
+				break;
+			case I2C_DRIVERID_CX25840:
+				print_regs(fd, &reg, 0, 2, stride);
+				print_regs(fd, &reg, 0x100, 0x15f, stride);
+				print_regs(fd, &reg, 0x200, 0x23f, stride);
+				print_regs(fd, &reg, 0x400, 0x4bf, stride);
+				print_regs(fd, &reg, 0x800, 0x9af, stride);
+				break;
+			case 0:
+				print_regs(fd, &reg, 0x02000000, 0x0200ffff, stride);
+				break;
 			}
-			usleep(1);
 		}
-		printf("\n");
+		else {
+			print_regs(fd, &reg, min_reg, max_reg, stride);
+		}
 	}
 
 	if (options[OptFrameSync]) {
@@ -773,7 +777,7 @@ int main(int argc, char **argv)
 		new_debug_level = debug_level;
 
 		sprintf(buf, "%d", debug_level);
-		if (dowrite(buf, "/sys/module/ivtv/parameters/ivtv_debug") == 0) {
+		if (dowrite(buf, "/sys/module/ivtv/parameters/debug") == 0) {
 			printf(" set debug level: ");
 			print_debug_mask(new_debug_level);
 			printf("\n");
@@ -784,7 +788,7 @@ int main(int argc, char **argv)
 		char *buf;
 
 		gdebug_level = 0;
-		if ((buf = doread("/sys/module/ivtv/parameters/ivtv_debug"))) {
+		if ((buf = doread("/sys/module/ivtv/parameters/debug"))) {
 			gdebug_level = atol(buf);
 			printf(" debug level: ");
 			print_debug_mask(gdebug_level);
