@@ -242,7 +242,7 @@ static void usage(void)
 	printf("  -s, --set-standard=<num>\n");
 	printf("                     set the video standard to <num> [VIDIOC_S_STD]\n");
 	printf("                     <num> can be a numerical v4l2_std value, or it can be one of:\n");
-	printf("                     pal-X (X = B/G/H/N/Nc/I/D/K/M) or just 'pal' (V4L2_STD_PAL)\n");
+	printf("                     pal-X (X = B/G/H/N/Nc/I/D/K/M/60) or just 'pal' (V4L2_STD_PAL)\n");
 	printf("                     ntsc-X (X = M/J/K) or just 'ntsc' (V4L2_STD_NTSC)\n");
 	printf("                     secam-X (X = B/G/H/D/K/L/Lc) or just 'secam' (V4L2_STD_SECAM)\n");
 	printf("  --list-standards   display supported video standards [VIDIOC_ENUMSTD]\n");
@@ -553,7 +553,8 @@ static void find_controls(int fd)
 	int id;
 
 	while (ioctl(fd, VIDIOC_QUERYCTRL, &qctrl) == 0) {
-		if (qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS) {
+		if (qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS &&
+		    !(qctrl.flags & V4L2_CTRL_FLAG_DISABLED)) {
 			ctrl_str2id[name2var(qctrl.name)] = qctrl.id;
 			ctrl_id2str[qctrl.id] = name2var(qctrl.name);
 		}
@@ -563,12 +564,14 @@ static void find_controls(int fd)
 		return;
 	for (id = V4L2_CID_USER_BASE; id < V4L2_CID_LASTP1; id++) {
 		qctrl.id = id;
-		if (ioctl(fd, VIDIOC_QUERYCTRL, &qctrl) == 0)
+		if (ioctl(fd, VIDIOC_QUERYCTRL, &qctrl) == 0 &&
+		    !(qctrl.flags & V4L2_CTRL_FLAG_DISABLED))
 			ctrl_str2id[name2var(qctrl.name)] = qctrl.id;
 	}
 	for (qctrl.id = V4L2_CID_PRIVATE_BASE;
 			ioctl(fd, VIDIOC_QUERYCTRL, &qctrl) == 0; qctrl.id++) {
-		ctrl_str2id[name2var(qctrl.name)] = qctrl.id;
+		if (!(qctrl.flags & V4L2_CTRL_FLAG_DISABLED))
+			ctrl_str2id[name2var(qctrl.name)] = qctrl.id;
 	}
 }
 
@@ -778,6 +781,8 @@ static v4l2_std_id parse_pal(const char *pal)
 {
 	if (pal[0] == '-') {
 		switch (pal[1]) {
+			case '6':
+				return V4L2_STD_PAL_60;
 			case 'b':
 			case 'B':
 			case 'g':
@@ -1445,7 +1450,7 @@ int main(int argc, char **argv)
 			struct v4l2_ext_control ctrl = { 0 };
 
 			ctrl.id = ctrl_str2id[iter->first];
-			ctrl.value = atol(iter->second.c_str());
+			ctrl.value = strtol(iter->second.c_str(), NULL, 0);
 			if (V4L2_CTRL_ID2CLASS(ctrl.id) == V4L2_CTRL_CLASS_MPEG)
 				mpeg_ctrls.push_back(ctrl);
 			else
